module Trader
  class Account
    include MarketLoader

    attr_reader :backend

    def initialize(_backend, _session)
      @backend = _backend
      @session = _session
    end

    def using(_pair, _quote=nil, &_block)
      _pair = CurrencyPair.for_code _pair, _quote

      proxy_market = find_exact_market _pair
      proxy_market = find_compatible_market _pair if proxy_market.nil?
      backend.not_supported _pair if proxy_market.nil?

      proxy = AccountProxy.new self, proxy_market, _pair

      return proxy if _block.nil?
      _block.call proxy
    end

    def using_default_pair(&_block)
      using available_markets.first, &_block
    end

    def balance_for(_currency)
      _currency = Currency.for_code _currency
      ensure_supported_currency _currency

      # IDEA: cache balances by currency code.
      balance = Balance.new backend, session, _currency
      balance.refresh!
      balance
    end

    def create_order(_order)
      ensure_supported_market _order.pair
      raw_order = backend.create_order(
        session,
        _order.pair,
        _order.volume.amount,
        _order.limit? ? _order.price.amount : nil,
        _order.instruction
      )

      AccountOrder.new backend, session, raw_order
    end

    def find_order(_id)
      raw_order = backend.fetch_order session, _id
      # TODO: maybe Position.new(backend, session, Market.new(backend, raw_order.pair), raw_order)
      AccountOrder.new backend, session, raw_order
    end

    def list_orders
      # TODO.
    end

    private

    attr_reader :session

    def find_exact_market(_pair)
      available_markets.find { |p| p == _pair }
    end

    def find_compatible_market(_pair)
      available_markets.find { |p| p.compatible_with? _pair }
    end
  end
end