module Trader
  class AccountOrder
    PENDING = :pending
    OPEN = :open
    CLOSED = :closed
    CANCELED = :canceled

    def initialize(_backend, _session, _raw, _forced_pair=nil)
      @backend = _backend
      @session = _session
      @raw = _raw
      @forced_pair = _forced_pair
    end

    def convert_to(_pair, _quote=nil)
      forced_pair = CurrencyPair.for_code _pair, _quote

      return self if forced_pair == pair

      self.class.new backend, session, @raw, forced_pair
    end

    def id
      @raw.id
    end

    def pair
      forced_pair || original_pair
    end

    def original_pair
      @raw.pair
    end

    def status
      @raw.status
    end

    def limit?
      @raw.limit?
    end

    def instruction
      @raw.instruction
    end

    def price
      if limit?
        convert_quote original_pair.quote.pack @raw.price
      else
        nil
      end
    end

    # TODO: executed_price

    def volume
      convert_base original_pair.base.pack @raw.volume
    end

    def executed_volume
      convert_base original_pair.base.pack @raw.executed_volume
    end

    def pending_volume
      convert_base(volume - executed_volume)
    end

    def refresh!
      @raw = backend.fetch_order(session, id)
      self
    end

    def cancel!
      @raw = backend.cancel_order(session, id)
      self
    end

    def finished?
      status == CLOSED or status == CANCELED
    end

    private

    attr_reader :backend, :session, :forced_pair

    def convert_base(_price)
      return _price if forced_pair.nil?
      _price.convert_to forced_pair.base
    end

    def convert_quote(_price)
      return _price if forced_pair.nil?
      _price.convert_to forced_pair.quote
    end
  end
end