module Trader
  class Balance
    def initialize(_backend, _session, _currency, _forced_currency=nil)
      @backend = _backend
      @session = _session
      @currency = _currency
      @forced_currency = _forced_currency
      @raw = nil
    end

    def currency
      forced_currency || @currency
    end

    def original_currency
      @currency
    end

    def convert_to(_currency)
      return self if _currency == currency
      copy = self.class.new backend, session, original_currency, _currency
      copy.raw = raw unless raw.nil?
      copy
    end

    def amount
      convert original_currency.pack raw.amount
    end

    def available_amount
      convert original_currency.pack raw.available_amount
    end

    def frozen_amount
      convert(amount - available_amount)
    end

    def refresh!
      self.raw = backend.get_balance session, currency
      self
    end

    def new_deposit_endpoint
      backend.generate_endpoint session, original_currency
    end

    def withdraw(_amount, to: nil)
      _amount = currency.pack _amount
      _amount = _amount.convert_to original_currency

      backend.withdraw_to_endpoint session, currency, _amount.amount, to
      self
    end

    def transfer(_amount, to: nil)
      withdraw _amount, to: to.new_deposit_endpoint
    end

    protected

    attr_accessor :raw

    private

    attr_reader :backend, :session, :forced_currency

    def convert(_price)
      return _price if forced_currency.nil?
      _price.convert_to forced_currency
    end
  end
end