module Trader
  class BackendFactory
    def self.build(_backend)
      case _backend
      when String, Symbol
        self.new(_backend).build
      else
        _backend
      end
    end

    attr_reader :name

    def initialize(_name)
      @name = _name
    end

    def build
      builder = "build_#{name}"
      raise "Invalid backend #{name}" unless self.class.private_method_defined? builder
      send builder
    end

    private

    def build_game
      require 'trade-o-matic/adapters/game_backend'
      GameBackend.new
    end

    def build_bitstamp
      require 'trade-o-matic/adapters/bitstamp_backend'
      BitstampBackend.new
    end

    def build_bitfinex
      require 'trade-o-matic/adapters/bitfinex_backend'
      BitfinexBackend.new
    end

    def build_surbtc
      require 'trade-o-matic/adapters/surbtc_backend'
      SurbtcBackend.new
    end

    def build_itbit
      require 'trade-o-matic/adapters/itbit_backend'
      ItbitBackend.new
    end

    def build_fake
      require 'trade-o-matic/adapters/fake_backend'
      FakeBackend.instance
    end
  end
end
