module Trader
  class Book
    attr_reader :pair, :bid_slope, :ask_slope

    def initialize(_pair)
      @pair = _pair
      @bid_slope = BidSlope.new _pair
      @ask_slope = AskSlope.new _pair
    end

    def prepare(_ref_time=nil)
      @ref_time = _ref_time || Time.now
      @transactions = []
      @bid_slope.reset
      @ask_slope.reset
    end

    def add_transaction(_price, _volume, _time)
      @transactions << Transaction.new(
        Price.new(pair.quote, _price),
        Price.new(pair.base, _volume),
        _time
      )
      nil
    end

    def add_bid(_price, _volume)
      bid_slope.add_order _price, _volume
    end

    def add_ask(_price, _volume)
      ask_slope.add_order _price, _volume
    end

    def volume(_period_seconds)
      ref_time = @ref_time - _period_seconds
      amount = @transactions.inject(0.0) do |r, trx|
        if trx.time >= ref_time
          r + trx.volume.amount
        else
          r
        end
      end

      Price.new pair.base, amount
    end

    private

    attr_reader :transactions
  end
end