module Trader
  class Price
    attr_reader :currency, :amount

    def initialize(_currency, _amount)
      @currency = Currency.for_code _currency
      @amount = Standard.amount _amount
    end

    def pack(_amount)
      currency.pack _amount
    end

    def convert_to(_other_currency)
      _other_currency = Currency.for_code _other_currency

      return self if _other_currency == currency

      new_amount = @currency.convert amount, _other_currency
      Price.new _other_currency, new_amount
    end

    def to_s
      "#{amount} #{currency.code}"
    end

    [:<, :<=, :>=, :>, :==, '<=>'].each do |op|
      define_method(op) do |_other|
        _other = currency.unpack(_other, strict: false)
        amount.public_send(op, _other)
      end
    end

    [:+, :-, :*, :/].each do |op|
      define_method(op) do |_other|
        _other = currency.unpack(_other, strict: false)
        Price.new currency, amount.public_send(op, _other)
      end
    end
  end
end