module Trader
  class Slope
    attr_reader :pair

    def initialize(_pair)
      @pair = _pair
      reset
    end

    def reset
      @orders = []
      @orders_sorted = false
    end

    def add_order(_price, _volume)
      @orders_sorted = false
      @orders << Order.new(pair, instruction, _volume, _price)
      nil
    end

    def price(_skip_volume=nil)
      _skip_volume = pair.base.unpack _skip_volume, 0.0

      slice(_skip_volume).first[0].price
    end

    def volume_for_price(_price, _skip_volume=nil)
      _price = pair.quote.unpack _price
      _skip_volume = pair.base.unpack _skip_volume, 0.0

      wt_price = 0.0
      r_volume = 0.0

      slice(_skip_volume).each do |order, volume|
        if compare(order.price.amount, _price) == -1
          x = (_price - wt_price) * r_volume / (order.price.amount - _price)
          if x < volume
            volume = x
          end
        end

        wt_price = (wt_price * r_volume + order.price.amount * volume) / (r_volume + volume)
        r_volume += volume
      end

      Price.new pair.base, r_volume
    end

    def quote(_volume, _skip_volume=nil)
      _volume = pair.base.unpack _volume, 0.0
      _skip_volume = pair.base.unpack _skip_volume, 0.0

      total_price = 0.0
      slice(_skip_volume, _volume).each do |order, order_volume|
        total_price += order_volume * order.price.amount
      end

      Price.new pair.quote, total_price
    end

    def assess(_amount, _skip_volume=nil)
      _amount = pair.quote.unpack _amount
      _skip_volume = pair.base.unpack _skip_volume, 0.0

      total_volume = 0.0
      slice(_skip_volume).each do |order, volume|
        order_quote = order.price.amount * volume
        if order_quote > _amount
          total_volume += _amount / order.price.amount
          break
        else
          total_volume += volume
          _amount -= order_quote
        end
      end

      Price.new pair.base, total_volume
    end

    private

    def orders
      unless @orders_sorted
        sort_by_relevance @orders
        @orders_sorted = true
      end

      @orders
    end

    def slice(_skip_volume, _volume=nil)
      return to_enum(:slice, _skip_volume, _volume) unless block_given?

      orders.each do |order|
        if order.volume.amount >= _skip_volume
          volume = order.volume.amount - _skip_volume

          unless _volume.nil?
            volume = _volume if _volume < volume
            _volume -= volume
          end

          yield order, volume
          break if _volume == 0.0
          _skip_volume = 0.0
        else
          _skip_volume -= order.volume.amount
        end
      end
    end

    def sort_by_relevance(_orders)
      _orders.sort! { |a, b| compare a.price.amount, b.price.amount }
    end

    def order_type
      # abstract
    end

    def compare(_price_a, _price_b)
      # abstract
    end
  end
end