module Trader
  module ConverterConfigurator
    extend self

    def from_yaml(_yaml)
      _yaml.each do |conv|
        type = conv['type'] || detect_type(conv)
        converter = send "configure_yaml_#{type}", conv

        Currency.register_conversion conv['from'], conv['to'], converter, inverse: !!conv['inverse']
        Currency.register_conversion conv['to'], conv['from'], converter, inverse: !conv['inverse'] if conv['symetric']
      end
    end

    private

    def configure_yaml_fixed(_params)
      FixedConverter.new _params['rate']
    end

    def configure_yaml_json_api(_params)
      require 'trade-o-matic/converters/json_api_converter'
      JsonApiConverter.new(_params['url'], _params['path'].split('.'), _params['ttl'] || 30)
    end

    def configure_yaml_web(_params)
      require 'trade-o-matic/converters/web_converter'
      WebConverter.new(_params['url'], _params['path'], _params['ttl'] || 30)
    end

    def invert(_converter)
      InverseConverter.new _converter
    end

    def detect_type(_params)
      return 'json_api' if _params['url']
      return 'fixed'
    end
  end
end